// ListCtrlHelper.cpp: Implementierung der Klasse CListCtrlHelper.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "ListCtrlHelper.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CListCtrlHelper::CListCtrlHelper(CListCtrl *lc, HINSTANCE hinstance) :
	m_plc(lc), m_hinstance(hinstance)
{
	m_images = lc->GetImageList(LVSIL_SMALL);
}

CListCtrlHelper::~CListCtrlHelper()
{

}

CString CListCtrlHelper::GetSmartFileSize(unsigned int filesize)
{
	CString result;
	char append = ' ';
	if (filesize < KILO)
	{
		result.Format("%u", filesize);
		return result;
	}
	if (filesize < MEGA)
		result.Format("%.0uK", filesize / KILO);
	else if (filesize < GIGA)
		result.Format("%.1fM", (float)filesize / MEGA);
	else
		result.Format("%.2fG", (float)filesize / GIGA);
	return result;
}

BOOL CListCtrlHelper::ListControlToFile(CStdioFile &file, unsigned int first)
{
	CString line, buf;
	LVITEM item;

	line.Format("%u\n", first);
	file.WriteString(line);

	for (int i = 0; i < m_plc->GetItemCount(); i++)
	{
		line.Empty();
		item.mask = LVIF_IMAGE;
		item.iItem = i;
		item.iSubItem = 0;
		m_plc->GetItem(&item);
		line.Format("%d*", item.iImage);

		for (int j = 0; j < 4; j++)
		{
			m_plc->GetItemText(i, j, buf.GetBuffer(1024), 1024);
			buf.ReleaseBuffer(-1);
			line += buf + '*';
		}
		file.WriteString(line + '\n');
	}
	return TRUE;
}

BOOL CListCtrlHelper::FileToListControl(int num, unsigned int &first)
{
	ASSERT(num >= 0);

	CString filename, line;
	CStdioFile file;

	m_plc->DeleteAllItems();
	filename.Format("%06d", num);
	if (!file.Open(filename, CFile::modeRead))
	{
		AfxMessageBox("Fatal Error! Could not open cache file");
		return FALSE;
	}

	// read number of first entry
	file.ReadString(line);
	first = atoi(line);

	int i = 0;
	while (file.ReadString(line))
	{
		int pos0 = line.Find('*');
		int pos1 = line.Find('*', pos0 + 1);
		int pos2 = line.Find('*', pos1 + 1);
		int pos3 = line.Find('*', pos2 + 1);
		int pos4 = line.Find('*', pos3 + 1);
		int index = m_plc->InsertItem(i, line.Mid(pos0+1, pos1-pos0-1), 
										  atoi(line.Left(pos0)));
		
		m_plc->SetItem(index, 1, LVIF_TEXT, line.Mid(pos1+1, pos2-pos1-1), -1, 0, 0, 0);
		
		m_plc->SetItem(index, 2, LVIF_TEXT, line.Mid(pos2+1, pos3-pos2-1), -1, 0, 0, 0);
		m_plc->SetItem(index, 3, LVIF_TEXT, line.Mid(pos3+1, pos4-pos3-1), -1, 0, 0, 0);
		i++;
	}
	return TRUE;

}

void CListCtrlHelper::DeleteCache(CString path, int num)
{
	CString format;
	for (int i = 0; i <= num; i++)
	{
		try
		{
			format.Format("%06d", i);
			CFile::Remove(path + format);
		}
		catch (CFileException *e) { e->GetErrorMessage(format.GetBuffer(1000), 1000);format.ReleaseBuffer(-1);AfxMessageBox(format);e->Delete(); }
	}
}

int CListCtrlHelper::RetrieveURLs(URL_ARRAY &urls)
{
	POSITION selectedItems = m_plc->GetFirstSelectedItemPosition();
	//ASSERT(selectedItems != NULL);
	while (selectedItems)
	{
		int nItem = m_plc->GetNextSelectedItem(selectedItems);

		urls.Add(EntryToUNC(nItem));
		//AfxMessageBox(url);
	}
	return urls.GetSize();
}

void CListCtrlHelper::SetRegistryValue(HKEY root, CString path, CString key, CString value)
{
	CRegKey reg;

	if ( reg.Open(root, "") != ERROR_SUCCESS )
	{
		return;
	}
	reg.SetKeyValue((LPCTSTR)path, (LPCTSTR)value, (LPCTSTR)key);
}

void CListCtrlHelper::GetRegistryValue(HKEY root, CString path, CString key, CString &value)
{
	CRegKey reg;

	if ( reg.Open(root, path) == ERROR_SUCCESS )
	{
		DWORD length = 255;
		CString value;
		if ( reg.QueryValue(value.GetBuffer(255), key, &length) == ERROR_SUCCESS )
			value.ReleaseBuffer(length);
		else
			value = "";
	}
	else
		value = "";

}

int CListCtrlHelper::GetIndexIcon(CString &filename)
{
	CString extension(filename.Right(filename.GetLength() - filename.ReverseFind('.')));
	int position;
	if ( m_map.Lookup(extension, position) )
	{
		return position;
	}
	else
	{
		CRegKey reg;
		unsigned long buf_length = 5012;
		char buf[5012];
		memset(buf, 0, buf_length);
		
		if ( reg.Open(HKEY_CLASSES_ROOT, extension) == ERROR_SUCCESS &&
			 reg.QueryValue(buf, "", &buf_length) == ERROR_SUCCESS )
		{
			CString string(buf);
			reg.Close();
			unsigned long buf_length = 5012;
			if ( reg.Open(HKEY_CLASSES_ROOT, string + "\\DefaultIcon") == ERROR_SUCCESS &&
				 reg.QueryValue(buf, "", &buf_length) == ERROR_SUCCESS )
			{
				string = buf;
				if ( string == "%1" )
					return ICON_UNKNOWN;
				CString path = string.Left(string.ReverseFind(','));
				
				WORD num = atoi(string.Right(string.GetLength() - string.ReverseFind(',') - 1));
				HICON icon = ExtractAssociatedIcon(m_hinstance, path.GetBuffer(4096), &num);
				path.ReleaseBuffer(-1);
				reg.Close();
				int result = m_images->Add(icon);
				m_map.SetAt(extension, result);

				if ( result == -1)
					return ICON_UNKNOWN;
				else
					return result;
			}
			else
				return ICON_UNKNOWN;
		}
		else
			return ICON_UNKNOWN;
	}
}

void CListCtrlHelper::AddRow(int index, CString path, int icon_path, CString size,
							 CString host, CString type)
{
	int num = m_plc->InsertItem(LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM,
							    index, path, 0, 0, icon_path, (LPARAM)index);

	ASSERT ( num != -1 );
/*
	LVITEM lv;
	lv.mask = LVIF_TEXT | LVIF_PARAM;
	lv.iItem = num;
	lv.iSubItem = 1;
	lv.lParam = (LPARAM)((1 << 16) + index);
	BOOL a = m_plc->SetItem(&lv);*/
	if (!m_plc->SetItem(num, 1, LVIF_TEXT, size, -1, 0, 0, (LPARAM)((1 << 16) + index)) ||
		!m_plc->SetItem(num, 2, LVIF_TEXT, host, -1, 0, 0, (2 << 16) + index) ||
		!m_plc->SetItem(num, 3, LVIF_TEXT, type, -1, 0, 0, (3 << 16) + index))
		ASSERT(FALSE);		
}

BOOL CListCtrlHelper::ListControlToHTMLFile(CStdioFile &file, CString search,
											unsigned int first)
{
	CString line, buf;

	line.LoadString(IDS_HTML_START);
	file.WriteString(line);
	line.LoadString(IDS_TABLE_HEADER);
	file.WriteString(line);

	line.Format("\t<b>Suchergebnis fr \"%s\", Treffer %u - %u:</b>\n", search,
				first, first + m_plc->GetItemCount() - 1);
	file.WriteString(line);
	
	for (int i = 0; i < m_plc->GetItemCount(); i++)
	{
		file.WriteString("\t<tr>\n");
		line.Empty();
		for (int j = 0; j < 4; j++)
		{
			file.WriteString("\t\t<td>\n");
			m_plc->GetItemText(i, j, buf.GetBuffer(1024), 1024);
			buf.ReleaseBuffer(-1);
			if ( j == 0 )
			{
				line = EntryToUNC(i);
				buf = "<a href=\"" + line + "\">" + buf + "</a>";
			}
			file.WriteString("\t\t" + buf + "\n\t\t</td>\n");
		}
		file.WriteString("\t</tr>\n");
	}

	line.LoadString(IDS_HTML_END);
	file.WriteString(line);

	return TRUE;
}

CString CListCtrlHelper::EntryToUNC(int nItem)
{
	CString buffer, url;
	bool ftp;

	m_plc->GetItemText(nItem, 3, buffer.GetBuffer(10), 10);
	buffer.ReleaseBuffer(-1);

	ftp = (buffer == "FTP");

	url = (ftp) ? "ftp://" : "\\\\";
	
	m_plc->GetItemText(nItem, 2, buffer.GetBuffer(5012), 5012);
	buffer.ReleaseBuffer(-1);
	url += buffer + '/';
	
	m_plc->GetItemText(nItem, 0, buffer.GetBuffer(5012), 5012);
	buffer.ReleaseBuffer(-1);
	url += buffer;
	if (!ftp)
		url.Replace('/', '\\');
	else
		url.Replace("anonymous:user@host.com@", "");

	return url;
}

double CListCtrlHelper::GetFileSize(CString filesize)
{
	int factor = 1;

	if (filesize.Right(1).FindOneOf("KMG") == 0)
	{
		switch (filesize.Right(1)[0])
		{
		case 'K':
			factor = 1024;
			break;
		case 'M':
			factor = 1024*1024;
			break;
		case 'G':
			factor = 1024*1024*1024;
			break;
		}
		filesize.Delete(filesize.GetLength() - 1, 1);
	}
	
	return (factor*atof(filesize));
}